@@ -12,15 +12,7 @@
 (defclass stream-cipher (cipher)
   ())
 
-(defun encrypt (cipher plaintext ciphertext
-                &key (plaintext-start 0) plaintext-end
-                (ciphertext-start 0) handle-final-block)
-  "Encrypt the data in PLAINTEXT between PLAINTEXT-START and
-PLAINTEXT-END according to CIPHER.  Places the encrypted data in
-CIPHERTEXT, beginning at CIPHERTEXT-START.  Less data than
- (- PLAINTEXT-END PLAINTEXT-START) may be encrypted, depending on the
-alignment constraints of CIPHER and the amount of space available in
-CIPHERTEXT."
+(defmethod encrypt ((cipher cipher) plaintext ciphertext &key (plaintext-start 0) plaintext-end (ciphertext-start 0) handle-final-block &allow-other-keys)
   (check-type plaintext vector)
   (let ((plaintext-end (or plaintext-end (length plaintext))))
     (funcall (slot-value (mode cipher) 'encrypt-function)
@@ -28,15 +20,7 @@ CIPHERTEXT."
              plaintext-start plaintext-end ciphertext-start
              handle-final-block)))
 
-(defun decrypt (cipher ciphertext plaintext
-                &key (ciphertext-start 0) ciphertext-end
-                (plaintext-start 0) handle-final-block)
-  "Decrypt the data in CIPHERTEXT between CIPHERTEXT-START and
-CIPHERTEXT-END according to CIPHER.  Places the decrypted data in
-PLAINTEXT, beginning at PLAINTEXT-START.  Less data than
- (- CIPHERTEXT-END CIPHERTEXT-START) may be encrypted, depending on the
-alignment constraints of CIPHER and the amount of space available in
-PLAINTEXT."
+(defmethod decrypt ((cipher cipher) ciphertext plaintext &key (ciphertext-start 0) ciphertext-end (plaintext-start 0) handle-final-block &allow-other-keys)
   (check-type ciphertext vector)
   (let ((ciphertext-end (or ciphertext-end (length ciphertext))))
     (funcall (slot-value (mode cipher) 'decrypt-function)
@@ -190,9 +174,6 @@ PLAINTEXT."
      (declare (type index plaintext-start ciphertext-start length))
      ,@body))
 
-(defgeneric verify-key (cipher key)
-  (:documentation "Return T if KEY is a valid encryption key for CIPHER."))
-
 ;; Catch various errors.
 (defmethod verify-key (cipher key)
   ;; check the key first
@@ -204,10 +185,6 @@ PLAINTEXT."
   (unless (member cipher (list-all-ciphers))
     (error 'unsupported-cipher :name cipher)))
 
-(defgeneric schedule-key (cipher key)
-  (:documentation "Schedule KEY for CIPHER, filling CIPHER with any
-round keys, etc. needed for encryption and decryption."))
-
 (defmethod schedule-key :before ((cipher cipher) key)
   (verify-key cipher key))
 
@@ -230,13 +207,6 @@ round keys, etc. needed for encryption and decryption."))
 (defun (setf %find-cipher) (cipher-info name)
   (setf (get (massage-symbol name) '%cipher-info) cipher-info))
 
-(defgeneric key-lengths (cipher)
-  (:documentation "Return a list of possible lengths of a key for
-CIPHER.  CIPHER may either be a cipher name as accepted by
-MAKE-CIPHER or a cipher object as returned by MAKE-CIPHER.  NIL
-is returned if CIPHER does not name a known cipher or is not a
-cipher object."))
-
 (defmethod key-lengths (cipher)
   (let ((cipher-info (%find-cipher cipher)))
     (and cipher-info (%key-lengths cipher-info))))
@@ -244,13 +214,6 @@ cipher object."))
 (defmethod key-lengths ((cipher cipher))
   (key-lengths (class-name (class-of cipher))))
 
-(defgeneric block-length (cipher)
-  (:documentation "Return the number of bytes in an encryption or
-decryption block for CIPHER.  CIPHER may either be a cipher name
-as accepted by MAKE-CIPHER or a cipher object as returned by
-MAKE-CIPHER.  NIL is returned if CIPHER does not name a known
-cipher or is not a cipher object."))
-
 (defmethod block-length ((cipher symbol))
   (let ((cipher-info (%find-cipher (massage-symbol cipher))))
     (and cipher-info (%block-length cipher-info))))
