@@ -638,19 +638,17 @@
   (let ((state (sosemanuk-state context))
         (state-r (sosemanuk-state-r context))
         (keystream-buffer (sosemanuk-keystream-buffer context))
-        (keystream-buffer-remaining (sosemanuk-keystream-buffer-remaining context))
-        (remaining-keystream (make-array 80 :element-type '(unsigned-byte 8))))
+        (keystream-buffer-remaining (sosemanuk-keystream-buffer-remaining context)))
     (declare (type (simple-array (unsigned-byte 32) (*)) state state-r)
-             (type (simple-array (unsigned-byte 8) (80)) keystream-buffer remaining-keystream)
-             (type (integer 0 80) keystream-buffer-remaining)
-             (dynamic-extent remaining-keystream))
+             (type (simple-array (unsigned-byte 8) (80)) keystream-buffer)
+             (type (integer 0 80) keystream-buffer-remaining))
     (unless (zerop length)
       (unless (zerop keystream-buffer-remaining)
         (let ((size (min length keystream-buffer-remaining)))
           (declare (type (integer 0 80) size))
-          (replace remaining-keystream keystream-buffer
-                   :end1 size :start2 (- 80 keystream-buffer-remaining))
-          (xor-block size remaining-keystream plaintext plaintext-start ciphertext ciphertext-start)
+          (xor-block size keystream-buffer (- 80 keystream-buffer-remaining)
+                     plaintext plaintext-start
+                     ciphertext ciphertext-start)
           (decf keystream-buffer-remaining size)
           (decf length size)
           (incf ciphertext-start size)
@@ -659,10 +657,10 @@
         (loop
           (sosemanuk-compute-block state state-r keystream-buffer)
           (when (<= length 80)
-            (xor-block length keystream-buffer plaintext plaintext-start ciphertext ciphertext-start)
+            (xor-block length keystream-buffer 0 plaintext plaintext-start ciphertext ciphertext-start)
             (setf (sosemanuk-keystream-buffer-remaining context) (- 80 length))
             (return-from sosemanuk-crypt (values)))
-          (xor-block 80 keystream-buffer plaintext plaintext-start ciphertext ciphertext-start)
+          (xor-block 80 keystream-buffer 0 plaintext plaintext-start ciphertext ciphertext-start)
           (decf length 80)
           (incf ciphertext-start 80)
           (incf plaintext-start 80)))
