@@ -62,16 +62,11 @@
   (flet ((stuff-state (state digest start)
            (declare (type (simple-array (unsigned-byte 8) (*)) digest))
            (declare (type (integer 0 #.(- array-dimension-limit 4)) start))
-           (setf (nibbles:ub32ref/be digest start)
+           (setf (ub32ref/be digest start)
                  (logior (ash (adler32-s2 state) 16)
                          (adler32-s1 state)))
            digest))
     (declare (inline stuff-state))
-    (etypecase digest
-      ((simple-array (unsigned-byte 8) (*))
-       (stuff-state state digest digest-start))
-      (null
-       (stuff-state state
-                    (make-array 4 :element-type '(unsigned-byte 8)) 0)))))
+    (stuff-state state digest digest-start)))
 
 (defdigest adler32 :digest-length 4 :block-length 1)
