@@ -252,18 +252,6 @@
 ;;; familiar digest interface below, but these are likely to be slightly
 ;;; more efficient, as well as more obvious about what you're trying to
 ;;; do.
-(defgeneric digest-file (digest-spec pathname &rest args
-                                     &key buffer start end
-                                     digest digest-start)
-  (:documentation "Return the digest of the contents of the file named by PATHNAME using
-the algorithm DIGEST-NAME.
-
-If DIGEST is provided, the digest will be placed into DIGEST starting at
-DIGEST-START.  DIGEST must be a (SIMPLE-ARRAY (UNSIGNED-BYTE 8) (*)).
-An error will be signaled if there is insufficient room in DIGEST.
-
-If BUFFER is provided, the portion of BUFFER between START and END will
-be used to hold data read from the stream."))
 
 (defmethod digest-file ((digest-name cons) pathname &rest kwargs)
   (apply #'digest-file (apply #'make-digest digest-name) pathname kwargs))
@@ -279,19 +267,6 @@ be used to hold data read from the stream."))
                                :buffer buffer :start start :end end)
     (produce-digest state :digest digest :digest-start digest-start)))
 
-(defgeneric digest-stream (digest-spec stream &rest args
-                                       &key buffer start end
-                                       digest digest-start)
-  (:documentation "Return the digest of the contents of STREAM using the algorithm
-DIGEST-NAME.  STREAM-ELEMENT-TYPE of STREAM should be (UNSIGNED-BYTE 8).
-
-If DIGEST is provided, the digest will be placed into DIGEST starting at
-DIGEST-START.  DIGEST must be a (SIMPLE-ARRAY (UNSIGNED-BYTE 8) (*)).
-An error will be signaled if there is insufficient room in DIGEST.
-
-If BUFFER is provided, the portion of BUFFER between START and END will
-be used to hold data read from the stream."))
-
 (defmethod digest-stream ((digest-name cons) stream &rest kwargs)
   (apply #'digest-stream (apply #'make-digest digest-name) stream kwargs))
 (defmethod digest-stream ((digest-name symbol) stream &rest kwargs)
@@ -303,18 +278,6 @@ be used to hold data read from the stream."))
                                :buffer buffer :start start :end end)
   (produce-digest state :digest digest :digest-start digest-start))
 
-(defgeneric digest-sequence (digest-spec sequence &rest args
-                                         &key start end digest digest-start)
-  (:documentation "Return the digest of the subsequence of SEQUENCE
-specified by START and END using the algorithm DIGEST-NAME.  For CMUCL
-and SBCL, SEQUENCE can be any vector with an element-type
-of (UNSIGNED-BYTE 8); for other implementations, SEQUENCE must be a
-(SIMPLE-ARRAY (UNSIGNED-BYTE 8) (*)).
-
-If DIGEST is provided, the digest will be placed into DIGEST starting at
-DIGEST-START.  DIGEST must be a (SIMPLE-ARRAY (UNSIGNED-BYTE 8) (*)).
-An error will be signaled if there is insufficient room in DIGEST."))
-
 (defmethod digest-sequence ((digest-name symbol) sequence &rest kwargs)
   (apply #'digest-sequence (make-digest digest-name) sequence kwargs))
 
@@ -352,24 +315,6 @@ An error will be signaled if there is insufficient room in DIGEST."))
            (error 'unsupported-digest :name digest-name))))
     (t
      (error 'type-error :datum digest-name :expected-type 'symbol))))
-
-(defgeneric copy-digest (digester &optional copy)
-  (:documentation "Return a copy of DIGESTER.  If COPY is not NIL, it
-should be of the same type as DIGESTER and will receive the copied data,
-rather than creating a new object.  The copy is a deep copy, not a
-shallow copy as might be returned by COPY-STRUCTURE."))
-
-(defgeneric update-digest (digester thing &key &allow-other-keys)
-  (:documentation "Update the internal state of DIGESTER with THING.
-The exact method is determined by the type of THING."))
-
-(defgeneric produce-digest (digester &key digest digest-start)
-  (:documentation "Return the hash of the data processed by
-DIGESTER so far.
-
-If DIGEST is provided, the hash will be placed into DIGEST starting at
-DIGEST-START.  DIGEST must be a (SIMPLE-ARRAY (UNSIGNED-BYTE 8) (*)).
-An error will be signaled if there is insufficient room in DIGEST."))
  
 
 ;;; the digest-defining macro
@@ -388,9 +333,6 @@ An error will be signaled if there is insufficient room in DIGEST."))
   (and (symbolp name)
        (not (null (digestp name)))))
 
-(defgeneric digest-length (digest)
-  (:documentation "Return the number of bytes in a digest generated by DIGEST."))
-
 (defmethod digest-length ((digest-name symbol))
   (or (digestp (massage-symbol digest-name))
       (error 'unsupported-digest :name digest-name)))
